@@ -28,7 +28,9 @@ except ImportError:
     deferred_open = open
 from vermouth.file_writer import DeferredFileWriter
 from vermouth.citation_parser import citation_formatter
+from vermouth.graph_utils import make_residue_graph
 from polyply import (MetaMolecule, ApplyLinks, Monomer, MapToMolecule)
+from polyply.src.graph_utils import find_missing_edges
 from .load_library import load_library
 
 LOGGER = StyleAdapter(get_logger(__name__))
@@ -56,21 +58,42 @@ def split_seq_string(sequence):
         monomers.append(Monomer(resname=resname, n_blocks=n_blocks))
     return monomers
 
-def gen_params(args):
+def gen_params(name, outpath, inpath=None, lib=None, seq=None, seq_file=None):
+    """
+    Top level function for running the polyply parameter generation.
+    Parameters seq and seq_file are mutually exclusive. Set the other
+    to None. Of inpath and lib only one has to be given. Set the other
+    to None if not used.
+
+    Parameters
+    ----------
+    name: str
+        name of the molecule in the itp file
+    outpath: :class:`pathlib.Path`
+        file path for output file
+    inpath: list[pathlib.Path]
+        list of paths to files with input definitions
+    library: str
+        name of the force field library to use
+    seq: list[str]
+        list of strings with format "resname:#monomers"
+    seqf: :class:`pathlib.Path`
+        file path to valid sequence file (.json/.fasta/.ig/.txt)
+    """
     # Import of Itp and FF files
     LOGGER.info("reading input and library files",  type="step")
-    force_field = load_library(args.name, args.lib, args.inpath)
+    force_field = load_library(name, lib, inpath)
 
     # Generate the MetaMolecule
-    if args.seq:
+    if seq:
         LOGGER.info("reading sequence from command",  type="step")
-        monomers = split_seq_string(args.seq)
+        monomers = split_seq_string(seq)
         meta_molecule = MetaMolecule.from_monomer_seq_linear(monomers=monomers,
                                                              force_field=force_field,
-                                                             mol_name=args.name)
-    elif args.seq_file:
+                                                             mol_name=name)
+    elif seq_file:
         LOGGER.info("reading sequence from file",  type="step")
-        meta_molecule = MetaMolecule.from_sequence_file(force_field, args.seq_file, args.name)
+        meta_molecule = MetaMolecule.from_sequence_file(force_field, seq_file, name)
 
     # Do transformationa and apply link
     LOGGER.info("mapping sequence to molecule",  type="step")
@@ -80,11 +103,13 @@ def gen_params(args):
 
     # Raise warning if molecule is disconnected
     if not nx.is_connected(meta_molecule.molecule):
-        n_components = len(list(nx.connected_components(meta_molecule.molecule)))
-        msg = "You molecule consists of {:d} disjoint parts. Perhaps links were not applied correctly."
-        LOGGER.warning(msg, (n_components))
+        LOGGER.warning("Your molecule consists of disjoint parts."
+                       "Perhaps links were not applied correctly.")
+        msg = "Missing link between residue {idxA} {resA} and residue {idxB} {resB}"
+        for missing in find_missing_edges(meta_molecule, meta_molecule.molecule):
+            LOGGER.warning(msg, **missing)
 
-    with deferred_open(args.outpath, 'w') as outpath:
+    with deferred_open(outpath, 'w') as outfile:
         header = [ ' '.join(sys.argv) + "\n" ]
         header.append("Please cite the following papers:")
         for citation in meta_molecule.molecule.citations:
@@ -92,8 +117,8 @@ def gen_params(args):
             LOGGER.info("Please cite: " + cite_string)
             header.append(cite_string)
 
-        vermouth.gmx.itp.write_molecule_itp(meta_molecule.molecule, outpath,
-                                            moltype=args.name, header=header)
+        vermouth.gmx.itp.write_molecule_itp(meta_molecule.molecule, outfile,
+                                            moltype=name, header=header)
     DeferredFileWriter().write()
 
 # ducktape for renaming the itp tool
