@@ -18,9 +18,11 @@ Test that force field files are properly read.
 import textwrap
 import pytest
 import math
+import numpy as np
 import vermouth.forcefield
 import vermouth.molecule
 from vermouth.molecule import Interaction
+from vermouth.pdb.pdb import read_pdb
 import polyply.src.meta_molecule
 from polyply import TEST_DATA
 from polyply.src.topology import Topology
@@ -34,6 +36,14 @@ class TestTopology:
 
     @staticmethod
     def test_add_positions_from_gro():
+        """
+        This test checks if coordinates are supplied at the molecule level that
+        coordinates are transfered to the meta_mol.molecule nodes and that the
+        meta_molecule nodes have the COG coordinates of the positions. Furthermore,
+        the last residue has no coordinates defined. In this case these nodes have
+        no coordinates defined and require the build and backmap attribute to be True.
+        In all other cases they need to be False.
+        """
         top = Topology.from_gmx_topfile(TEST_DATA + "/topology_test/system.top", "test")
         top.add_positions_from_file(TEST_DATA + "/topology_test/test.gro")
         for node in top.molecules[0].molecule.nodes:
@@ -44,21 +54,63 @@ class TestTopology:
             if node != 2:
                 assert "position" in top.molecules[0].nodes[node].keys()
                 assert top.molecules[0].nodes[node]["build"] == False
+                assert top.molecules[0].nodes[node]["backmap"] == False
             else:
                 assert top.molecules[0].nodes[node]["build"] == True
+                assert top.molecules[0].nodes[node]["backmap"] == True
+
+    @staticmethod
+    def test_add_meta_positions_from_gro():
+        """
+        This test checks if coordinates are supplied at the meta_molecule level that
+        coordinates are transfered to the meta_mol nodes, but not the molecule nodes.
+        Furthermore, the last residue has no coordinates defined. In this case these nodes
+        have no coordinates defined at either level and require the build and backmap attribute
+        to be True. In all other cases the build attribute is False and backmap is True.
+        """
+        top = Topology.from_gmx_topfile(TEST_DATA + "/topology_test/system.top", "test")
+        top.add_positions_from_file(TEST_DATA + "/topology_test/meta.gro", resolution="meta_mol")
+        for node in top.molecules[0].nodes:
+            if node != 2:
+                assert "position" in top.molecules[0].nodes[node].keys()
+                assert top.molecules[0].nodes[node]["build"] == False
+                assert top.molecules[0].nodes[node]["backmap"] == True
+            else:
+                assert top.molecules[0].nodes[node]["build"] == True
+                assert top.molecules[0].nodes[node]["backmap"] == True
 
     @staticmethod
     def test_add_positions_from_pdb():
+        """
+        This test checks, if coordinates are supplied at the molecule level and no residues
+        are missing or skipped, that all molecule nodes have coordinates defined and that
+        the meta_molecule positions are defined as well. In addition all build and backmap
+        attributes have to be False.
+        """
         top = Topology.from_gmx_topfile(TEST_DATA + "/topology_test/pdb.top", "test")
         top.add_positions_from_file(TEST_DATA + "/topology_test/test.pdb")
-        for meta_mol in top.molecules:
+
+        pdb_mols = read_pdb(TEST_DATA + "/topology_test/test.pdb")
+        for idx, meta_mol in enumerate(top.molecules):
             for node in meta_mol.molecule.nodes:
-                assert "position" in meta_mol.molecule.nodes[node].keys()
+                ref_pos = pdb_mols[idx].nodes[node]['position']
+                assert np.all(ref_pos == meta_mol.molecule.nodes[node]['position'])
 
         for meta_mol in top.molecules:
             for node in meta_mol.nodes:
                     assert "position" in meta_mol.nodes[node].keys()
                     assert meta_mol.nodes[node]["build"] == False
+                    assert meta_mol.nodes[node]["backmap"] == False
+
+    @staticmethod
+    def test_add_positions_from_file_fail():
+        """
+        This test checks if coordinates for a residue at the molecule level
+        are incomplete the appropiate error is raised.
+        """
+        top = Topology.from_gmx_topfile(TEST_DATA + "/topology_test/system.top", "test")
+        with pytest.raises(IOError):
+            top.add_positions_from_file(TEST_DATA + "/topology_test/fail.gro")
 
     @staticmethod
     def test_convert_to_vermouth_system():
